const maybeUnhandledPromises = new WeakMap();
const pendingUnhandledRejections = [];
let lastPromiseId = 0;
let promiseRejectListeners = [];
global['addPromiseRejectListener'] = function(promiseRejectListener){
    promiseRejectListeners.push(promiseRejectListener)
}

function promiseRejectHandler(type, promise, reason) {
  console.log('type:' + type, ' reason: ' + reason);
  switch (type) {
    case 'kPromiseRejectWithNoHandler':
      unhandledRejection(promise, reason);
      break;
    case 'kPromiseHandlerAddedAfterReject':
      handledRejection(promise);
      break;
  }
}

function unhandledRejection(promise, reason) {
  maybeUnhandledPromises.set(promise, {
    reason,
    uid: ++lastPromiseId,
    warned: false
  });
  pendingUnhandledRejections.push(promise);

  Promise.resolve(1).then(value => 1).then(value=>{
    processPromiseRejections();
  })
}

function handledRejection(promise) {
  const promiseInfo = maybeUnhandledPromises.get(promise);
  if (promiseInfo !== undefined) {
    maybeUnhandledPromises.delete(promise);
  }
}

function processPromiseRejections() {
  let len = pendingUnhandledRejections.length;
  while (len--) {
    const promise = pendingUnhandledRejections.shift();
    const promiseInfo = maybeUnhandledPromises.get(promise);
    if (promiseInfo === undefined) {
      continue;
    }
    promiseInfo.warned = true;
    const { reason, uid } = promiseInfo;
    if(reason instanceof Error){
        console.log("unhandledRejection", reason.toString())
    }else{
        console.log("unhandledRejection", reason)
    }

    for(let rejectListener of promiseRejectListeners){
        if(typeof rejectListener == 'function'){
            rejectListener(promise, reason);
        }
    }
  }
}

